package acme;

public class AcmeCompany {
	Department m_acme;
	
	/**
	 * Founds ACME - m_acme points to a Department object representing ACME company
	 */
	public void found(){
		//creating all the workers:
		Worker Kipi = new Worker("Kipi");
		Worker Ugi = new Worker("Ugi");
		Worker Ufnik = new Worker("Ufnik");
		Worker Kermit = new Worker("Kermit");
		Worker MissPiggy = new Worker("Miss Piggy");
		Worker BipBip = new Worker("Bip Bip");
		Worker Willy = new Worker("Willy");
		Worker Rega = new Worker("Rega");
		Worker Dodli = new Worker("Dodli");
		Worker Micky = new Worker("Micky");
		Worker Donald = new Worker("Donald");
		Worker Pluto = new Worker("Pluto");
		//creating all the subdepartments:
		Department Finance = new Department("Finance");
		Department HR = new Department("HR");
		Department RnD = new Department("R&D");
		Department Expenses = new Department("Expenses");
		Department Income = new Department("Income");
		Department Road = new Department("Road");
		Department Coyotes = new Department("Coyotes");
		Department Paint = new Department("Paint");
		Department Glue = new Department("Glue");
		
		//creating the hierarchy:
		m_acme = new Department("ACME");
		m_acme.addSubDepartment(Finance);
		m_acme.addSubDepartment(HR);
		m_acme.addSubDepartment(RnD);
		
		Finance.addSubDepartment(Expenses);
		Finance.addSubDepartment(Income);
		HR.addSubDepartment(Road);
		HR.addSubDepartment(Coyotes);
		RnD.addSubDepartment(Paint);
		RnD.addSubDepartment(Glue);
		
		Expenses.addWorker(Kipi);
		Expenses.addWorker(Ugi);
		Expenses.addWorker(Ufnik);
		Income.addWorker(Kermit);
		Income.addWorker(MissPiggy);
		Road.addWorker(BipBip);
		Coyotes.addWorker(Willy);
		Paint.addWorker(Rega);
		Paint.addWorker(Dodli);
		Glue.addWorker(Micky);
		Glue.addWorker(Donald);
		Glue.addWorker(Pluto);
	}

	/**
	 * 					*** ADDED PROCEDURE ***
	 * @pre worker and newDep are initialized;
	 * @post worker's old department, if not equal to newDep, doesn't contain worker
	 * 		 worker.getDepartment() == newDep
	 * @param worker - the worker to transfer
	 * @param newDep - the department to transfer worker to
	 */
    private void moveWorker(Worker worker, Department newDep){
    	worker.setDepartment(newDep);
    	newDep.addWorker(worker);
    }
	
	/**
     * @pre ACME is founded and:
     * 		the worker Ugi is initialized and belongs to department Expenses
     * 		the department Income is initialized
     * @post Ugi doesn't belong to the old department, and belongs to Income
     */
	public void moveUgi(){
		Worker Ugi = findWorker("Ugi");
		Department Income = findDepartment("Income");
		moveWorker(Ugi, Income);
	}
	
	/**
	 *  					*** ADDED PROCEDURE ***
	 * @pre worker is initialized and belongs to some department
	 * @post worker doesn't belong to any department
	 * @param worker - worker to be removed
	 */
	private void fireWorker(Worker worker){
		worker.getDepartment().removeWorker(worker);
		worker.setDepartment(null);
	}
	
	/**
	 * @pre ACME is founded and:
	 * 		the worker Kermit is initialized and is a worker in some
	 * 		department of m_acme
	 * @post Kermit is fired - that is doesn't belong to any of the subdepartments
	 * of m_acme
	 */
	public void fireKermit(){
		Worker Kermit = findWorker("Kermit");
		fireWorker(Kermit);
	}
	
	/**
	 * @pre ACME is founded and:
	 * - Glue is initialized and is a subdepartment of some level of m_acme
	 * - Coyotes is initialized and is a subdepartment of some level of m_acme
	 * - Mickey, Donald and Pluto are workers in some subdepartments of m_acme
	 * @post Closes the Glue department, fires Miki and Donald, and moves Pluto
	 * to the Coyotes department.
	 */
	public void reorganize(){
		Worker Micky = findWorker("Micky");
		Worker Donald = findWorker("Donald");
		Worker Pluto = findWorker("Pluto");
		Department Coyotes = findDepartment("Coyotes");
		Department Glue = findDepartment("Glue");
		
		fireWorker(Micky);
		fireWorker(Donald);
		moveWorker(Pluto, Coyotes);
		Glue.getParentDepartment().removeSubDepartment(Glue);
	}

    /**
     * 					*** ADDED PROCEDURE ***
     * @pre a department named depName exists and is a subdepartment
     * of some level of m_acme - will be ensured since should only be called
     * by findDepartment, that already has this condition
     * @post returns the department named depName
     * @param depName - the department name to look for
     * @param currDep - the current department to check
     * @return the department named depName
     */
	private Department findDepHelper(String depName, Department currDep){
    	if (currDep.getName() == depName) return currDep;
    	else if (currDep.getSubDepartments().length > 0)
    		for(Department dep:currDep.getSubDepartments()){
    			Department possibleResult = findDepHelper(depName, dep);
    			if (possibleResult.getName() == depName) return possibleResult;
    		}
    	return m_acme;
    }
	
	/**
	 * @pre a department named depName exists and is a subdepartment
     * of some level of m_acme.
     * @post returns the department named departmentName
	 * @param departmentName - a string that is a name of some department in ACME
	 * company
	 * @return the department named departmentName
	 */
	private Department findDepartment(String departmentName){
		Department result = findDepHelper(departmentName, m_acme);
		return result;
	}
	
	private Worker GHOST = new Worker("");	//for inner use of findWorkerHelper
											//like an "empty" worker
	
    /**
     *  					*** ADDED PROCEDURE ***
     * @pre a worker named workerName exists and is in a subdepartment
     * of some level of m_acme - will be ensured since should only be called
     * by findWorker, that already has this condition
     * @post returns the worker named workerName
     * @param workerName - a string that is a name of some worker in ACME
     * company
     * @param currDep - the current department checked for containing the
     * worker named workerName
     * @return the worker named workerName
     */
	private Worker findWorkerHelper(String workerName, Department currDep){
    	if (currDep.getWorkers().length == 0){
    		if (currDep.getSubDepartments().length > 0)
    			for(Department dep:currDep.getSubDepartments()){
    				Worker possibleWorker = findWorkerHelper(workerName, dep);
    				if (possibleWorker != GHOST) return possibleWorker; 
    				}
    	}
    	else for(Worker worker:currDep.getWorkers())
    		if (worker.getName() == workerName) return worker;
    	return GHOST;
    }
	
	/**
	 * @pre a worker named workerName exists and is in a subdepartment
     * of some level of m_acme
     * @post returns the worker named workerName
	 * @param workerName - a string that is a name of some worker in ACME
     * company
	 * @return the worker named workerName
	 */
	private Worker findWorker(String workerName){
		return findWorkerHelper(workerName, m_acme);
	}
	
	/**
	 * Prints the structure of ACME in a formatted way.
	 * @param title printed before the ACME structure.
	 */
	public void print(String title){
		System.out.println(title);
		System.out.println("------------------------------------");
		if (m_acme != null){
			m_acme.print("");
		}else{
			System.out.println("NO COMPANY");
		}		
	}
	
	public static void main(String[] args) {
		AcmeCompany acme = new AcmeCompany();
		
		acme.found();
		acme.print("ACME is founded!!!");
		
		System.out.println(acme.findDepartment("Finance").getName());
		System.out.println(acme.findWorker("Bip Bip").getName());
		
		acme.moveUgi();
		acme.print("ACME after the moving of Ugi");
		
		acme.fireKermit();
		acme.print("ACME after the dismissal of Kermit");
		
		acme.reorganize();
		acme.print("ACME after reorganization");
	}
}	
