import java.io.Serializable;

public class Contact implements Serializable{
	
	private String name;
	private String email;
	private String telephone;
	private Address address;
	
	/**
	 * Constructor: creates a new contact
	 * @param name - name of the contact
	 * @param email - an e-mail, may be null
	 * @param telephone - a telephone number, may be null
	 * @param address - an address, may be null
	 * @throws EmptyNameException - in case the name is null/empty
	 */
	public Contact(String name, String email, String telephone,
				   String street, String city, String zip, String country)
		throws EmptyNameException{
		if ((name == null) || (name.equals(new String(""))))
			throw new EmptyNameException();
		this.name = name;
		this.email = email;
		this.telephone = telephone;
		this.address = new Address(street, city, zip, country); 
	}
	
	/**
	 * Constructor: creates a new contact from a String array
	 * @param input - String array that contains the input fields (name etc.)
	 * @throws EmptyNameException - in case the name is null/empty
	 */
	public Contact(String[] input) throws EmptyNameException {
		if ((input[0] == null) || (input[0].equals(new String(""))))
			throw new EmptyNameException();
		this.name = input[0];
		this.email = input[1];
		this.telephone = input[2];
		this.address = new Address(input[3], input[4], input[5], input[6]);
	}
	
	/**
	 * returns the name of the contact
	 * @return this.name
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * returns the e-mail of the contact
	 * @return this.email
	 */
	public String getEmail(){
		return email;
	}
	
	/**
	 * returns the telephone of the contact
	 * @return this.telephone
	 */
	public String getTelephone(){
		return telephone;
	}
	
	/**
	 * returns the address of the contact
	 * @return this.address
	 */
	public Address getAddress(){
		return address;
	}
	
	/**
	 * returns the string representation of the contact
	 */
	public String toString(){
		String result = new String(name + "; " +
								   email + "; " +
								   telephone + "; " +
								   address.toString());
		return result;
	}
}
