public interface Course {

	/** 
	 * returns the units of the course
	 * @post ($ret >= 1) && ($ret <= 5)
	 */
	public int units();

	/**
	 * returns the level of the course
	 * @post ($ret >= 1) && ($ret <= 3)
	 */
	public int level();

	/**
	 * returns the number of students
	 * @post ($ret >= 0) && ($ret <= this.maxNumStudents())
	 */
	public int numOfStudents();

	/**
	 * returns the maximum number of students allowed to be registered to the course
	 * @post $ret >= 0
	 */
	public int maxNumStudents();

	/**
	 * returns true iff s is registered to the course
	 * @pre s != null
	 * @post $ret =  s is registered to the course
	 */
	public boolean registered(Student s);

	/**
	 * registers s to the course
	 * @pre s != null
	 * @pre this.registered(s) == false
	 * @pre this.numOfStudents() < this.maxNumStudents()
	 * @post this.registered(s) == true
	 * @post this.numOfStudents() == $prev(this).numOfStudents() + 1
	 */
	public void register(Student s);

	/**
	 * unregisters s from the course
	 * @pre this.registered(s) == true
	 * @post this.registered(s) == false
	 * @post this.numOfStudents() == $prev(this).numOfStudents() - 1
	 */
	public void drop(Student s);

}
