import java.util.Iterator;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;

public class GUIAddressBookViewer {	

	private Shell shell = null;
	private Table table = null;	
	private Label name, email, phone, street, city, zipcode, country;
	
	private AddressBook book = new AddressBook(); //Default AddressBook
	private boolean changed = false;
	private String currentFileName = null;

	public static void main(String[] args) {
		Display display = Display.getDefault();		
		GUIAddressBookViewer app = new GUIAddressBookViewer();
		app.createShell() ;
	
		app.shell.open();
		while (!app.shell.isDisposed()) {
			if (!display.readAndDispatch()) display.sleep ();
		}
		display.dispose();		
    }
    
    private void createEditor(final Composite parent) {    
        table = new Table(parent, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);

        TableColumn nameColumn  = new TableColumn(table,SWT.NONE);
        nameColumn.setText("Name");
        TableColumn emailColumn = new TableColumn(table,SWT.NONE);
        emailColumn.setText("Email");
        TableColumn phoneColumn = new TableColumn(table,SWT.NONE);
        phoneColumn.setText("Phone Number");
      
        table.setHeaderVisible(true);
        
        int tableFullWidth = parent.getClientArea().width - table.getBorderWidth() * 2;
        nameColumn.setWidth((int)(tableFullWidth * 0.20));
        emailColumn.setWidth((int)(tableFullWidth * 0.40));
        // Implementation Note: It is more precise to set the width of phoneColumn 
        // to be "tableFullWidth - nameColumn.getWidth() - emailColumn.getWidth()" 
        // instead of (int)(fullWidth * 0.40)  
        phoneColumn.setWidth(tableFullWidth - nameColumn.getWidth() - emailColumn.getWidth());
     
        Composite form = new Composite(parent,SWT.BORDER);      
        form.setLayout(new GridLayout(2,false));

        Label nameLabel = new Label(form,SWT.NONE);
        nameLabel.setText("Name");
        nameLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
        name = new Label(form,SWT.SINGLE | SWT.BORDER);
        name.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
        

        Label emailLable = new Label(form,SWT.NONE);
    	emailLable.setText("Email");
    	emailLable.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	email = new Label(form,SWT.SINGLE | SWT.BORDER);
    	email.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label phoneLabel = new Label(form,SWT.NONE);
    	phoneLabel.setText("Phone");
    	phoneLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	phone = new Label(form,SWT.SINGLE | SWT.BORDER);
    	phone.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
      
    	Label streetLable = new Label(form,SWT.NONE);
    	streetLable.setText("Street Address");
    	streetLable.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	street = new Label(form,SWT.SINGLE | SWT.BORDER);
    	street.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label cityLabel = new Label(form,SWT.NONE);
    	cityLabel.setText("City");
    	cityLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	city = new Label(form,SWT.SINGLE | SWT.BORDER);
    	city.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
      
    	Label zipcodeLabel = new Label(form,SWT.NONE);
    	zipcodeLabel.setText("Zip Code");
    	zipcodeLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	zipcode = new Label(form,SWT.SINGLE | SWT.BORDER);
    	zipcode.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label countryLabel = new Label(form,SWT.NONE);
    	countryLabel.setText("Country");
    	countryLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	country = new Label(form,SWT.SINGLE | SWT.BORDER);
    	country.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
    	
    	Sash sash = new Sash(parent,SWT.HORIZONTAL);
    	
    	parent.setLayout(new FormLayout());
      
        	
    	form.pack();
    	Point fsize = form.computeSize(SWT.DEFAULT,SWT.DEFAULT);
      
    	FormData topData = new FormData();
    	final FormData sashData = new FormData();
    	FormData bottomData = new FormData();
      
      	topData.left      = new FormAttachment(0);
    	topData.right     = new FormAttachment(100);
    	topData.top       = new FormAttachment(0);
      	topData.bottom    = new FormAttachment(sash);
      
       	sashData.left     = new FormAttachment(0);
    	sashData.right    = new FormAttachment(100);
      	sashData.top      = new FormAttachment(100,-fsize.y);
       
      	bottomData.left   = new FormAttachment(0);
    	bottomData.right  = new FormAttachment(100);
    	bottomData.top    = new FormAttachment(sash);
    	bottomData.bottom = new FormAttachment(100);
      
    	table.setLayoutData(topData);
    	sash.setLayoutData(sashData);
    	form.setLayoutData(bottomData);
    
    	sash.addSelectionListener(new SelectionAdapter() {
    	    public void widgetSelected(SelectionEvent e) {
    	        sashData.top = new FormAttachment(0,e.y);
    	        parent.layout();
    	    }
    	});
      
    	table.setSelection(0);
    	
		table.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event e){
				selectedContactDisplayUpdate();
			}
		});

		table.addListener(SWT.MouseDoubleClick, new Listener(){
			public void handleEvent(Event e){
				editContact();
			}
		});
    }
    
	private void createShell() {
		shell = new Shell();		   
		shell.setText("Address Book");
		
		Rectangle monitor_bounds = shell.getMonitor().getBounds();
		shell.setSize(new Point(monitor_bounds.width/3,
		                        monitor_bounds.height/2));
	    
	    Menu menu = new Menu(shell,SWT.BAR);
	    shell.setMenuBar(menu);
	    
	    MenuItem fileItem     = new MenuItem(menu,SWT.CASCADE);
	    MenuItem contactsItem = new MenuItem(menu,SWT.CASCADE);
	    
	    fileItem.setText("&File");
	    contactsItem.setText("&Contacts");
	    
	    Menu fileMenu = new Menu(shell,SWT.DROP_DOWN);
	    fileItem.setMenu(fileMenu);
	    Menu contactsMenu = new Menu(shell,SWT.DROP_DOWN);
	    contactsItem.setMenu(contactsMenu);

	    MenuItem newItem = new MenuItem(fileMenu,SWT.PUSH); 
	    newItem.setText("New &Address Book\tCtrl+N");
	    newItem.setAccelerator(SWT.MOD1+'N');
	    newItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				confirmSaveChanges();
	    				createNew();
	    			}
	    		});

	    MenuItem openItem = new MenuItem(fileMenu,SWT.PUSH); 
	    openItem.setText("&Open\tCtrl+O");
	    openItem.setAccelerator(SWT.MOD1+'O');
	    openItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				confirmSaveChanges();
	    				openBook();
	    			}
	    		});

	    MenuItem saveItem = new MenuItem(fileMenu,SWT.PUSH);
	    saveItem.setText("&Save\tCtrl+S");
	    saveItem.setAccelerator(SWT.MOD1+'S');
	    saveItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				saveBook();
	    			}
	    		});
	    
	    MenuItem exitItem = new MenuItem(fileMenu,SWT.PUSH);
	    exitItem.setText("E&xit\tCtrl+Q");
	    exitItem.setAccelerator(SWT.MOD1+'Q');
	    exitItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				confirmSaveChanges();
	    				confirmExit();
	    			}
	    		});
	    
	    MenuItem newContactItem = new MenuItem(contactsMenu,SWT.PUSH); 
	    newContactItem.setText("New &Contact\tCtrl+M");
	    newContactItem.setAccelerator(SWT.MOD1+'M');
	    newContactItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				newContact();
	    				displayUpdate();
	    			}
	    		});
	      
	    final MenuItem editItem = new MenuItem(contactsMenu,SWT.PUSH); 
	    editItem.setText("&Edit");
	    editItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				editContact();
	    			}
	    		});

	    final MenuItem deleteItem = new MenuItem(contactsMenu,SWT.PUSH); 
	    deleteItem.setText("&Delete");
	    deleteItem.setAccelerator(SWT.DEL);
	    deleteItem.addSelectionListener(
	    		new SelectionAdapter(){
	    			public void widgetSelected(SelectionEvent e){
	    				deleteContact();
	    			}
	    		});

	    createEditor(shell);
	}

	//  AddressBook operations:
	//  ***********************
	
	private void createNew(){
		this.book = new AddressBook();
		this.changed = false;
		this.currentFileName = null;
		displayUpdate();
	}
	
	private void openBook(){
		FileDialog openDialog = new FileDialog(shell, SWT.OPEN);
		openDialog.setText("Open Address Book");
		String filename = openDialog.open(); 
			
		try {
			if (filename != null){
				book.load(filename);
				changed = false;
				currentFileName = filename;
				displayUpdate();
				selectedContactDisplayUpdate();
			}
		} catch (Exception e){
			FileErrorWarning("open");
		}
	}
	
	private void saveBook(){
		if (changed){
		if (!(currentFileName == null)){
			try {
				book.save(currentFileName);
			} catch (Exception e){
				FileErrorWarning("save");
			}
		} else {
			FileDialog saveDialog = new FileDialog(shell, SWT.SAVE);
			saveDialog.setText("Save Address Book");
			String filename = saveDialog.open();
			
			try {
				if (filename != null){
					book.save(filename);
					changed = false;
					currentFileName = filename;
				}
			} catch (Exception e){
				FileErrorWarning("save");
			}
		}}
	}
	
	//  Display:
	//  ********
	
	private void displayUpdate() {
		table.removeAll();
		Iterator<Contact> iter = book.getContacts();
		while (iter.hasNext()) {
			Contact c = iter.next();
			TableItem item = new TableItem(table,SWT.NONE);
			String[] contact = new String[] {c.getName(),c.getEmail(),c.getTelephone()};
			item.setText(contact);
		}
		selectedContactDisplayUpdate();
	}

	private void selectedContactDisplayUpdate(){
		if (table.getItemCount() != 0){
			if (table.getSelectionIndex() == -1) {
				table.select(0); //if no selection has been made, selects first row as default
			}
			String selectedName = table.getSelection()[0].getText(0);
			try { //must wrap with try-catch due to use of class Contact
				Contact contact = book.get(selectedName);
				name.setText(contact.getName());
				phone.setText(contact.getTelephone());
				email.setText(contact.getEmail());
				street.setText(contact.getAddress().getStreet());
				city.setText(contact.getAddress().getCity());
				zipcode.setText(contact.getAddress().getZip());
				country.setText(contact.getAddress().getCountry());
			} catch (Exception e) {}
		} else {
			name.setText("");
			phone.setText("");
			email.setText("");
			street.setText("");
			city.setText("");
			zipcode.setText("");
			country.setText("");
		}
	}
	
	//  Contacts Operations:
	//  ********************
	
	private void ContactDetails(Contact c){
		Display localDisplay = Display.getCurrent();
		final Shell localShell = new Shell(localDisplay);
		localShell.setSize(new Point(localShell.getMonitor().getBounds().width/3,
						localShell.getMonitor().getBounds().height/4));
		localShell.setText("Edit a contact");
		localShell.setLayout(new GridLayout(2,false));
		final Contact contact = c;
		

        Label nameLabel = new Label(localShell ,SWT.NONE);
        nameLabel.setText("Name");
        nameLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
        final Text name = new Text(localShell , SWT.BORDER);
        name.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
        

        Label emailLable = new Label(localShell ,SWT.NONE);
    	emailLable.setText("Email");
    	emailLable.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text email = new Text(localShell , SWT.BORDER);
    	email.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label phoneLabel = new Label(localShell ,SWT.NONE);
    	phoneLabel.setText("Phone");
    	phoneLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text phone = new Text(localShell , SWT.BORDER);
    	phone.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
      
    	Label streetLable = new Label(localShell ,SWT.NONE);
    	streetLable.setText("Street Address");
    	streetLable.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text street = new Text(localShell , SWT.BORDER);
    	street.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label cityLabel = new Label(localShell ,SWT.NONE);
    	cityLabel.setText("City");
    	cityLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text city = new Text(localShell , SWT.BORDER);
    	city.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
      
    	Label zipcodeLabel = new Label(localShell ,SWT.NONE);
    	zipcodeLabel.setText("Zip Code");
    	zipcodeLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text zipcode = new Text(localShell , SWT.BORDER);
    	zipcode.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));

    	Label countryLabel = new Label(localShell ,SWT.NONE);
    	countryLabel.setText("Country");
    	countryLabel.setLayoutData(new GridData(GridData.BEGINNING,GridData.CENTER,false,false));
    	final Text country = new Text(localShell , SWT.BORDER);
    	country.setLayoutData(new GridData(GridData.FILL,GridData.CENTER,true,false));
    	
    	Composite buttonsComp = new Composite(localShell, SWT.BORDER);
    	buttonsComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 2, 1));
    	buttonsComp.setLayout(new FillLayout());
    	
    	Button applyButton = new Button(buttonsComp, SWT.NONE);
    	applyButton.setText("Apply");
    	//selectionListener will be added later
    	
    	Button cancelButton = new Button(buttonsComp, SWT.NONE);
    	cancelButton.setText("Cancel");
		cancelButton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				localShell.close();
			}
		});
		
    	//filling the text objects, in case of editing an existing contact:
    	if (contact != null){
    		name.setText(contact.getName());
    		email.setText(contact.getEmail());
    		phone.setText(contact.getTelephone());
    		street.setText(contact.getAddress().getStreet());
    		city.setText(contact.getAddress().getCity());
    		zipcode.setText(contact.getAddress().getZip());
    		country.setText(contact.getAddress().getCountry());
    	}
    	
    	applyButton.addSelectionListener(new SelectionAdapter(){
    		public void widgetSelected(SelectionEvent e){
    			try{ //must wrap all with try-catch due to use of class Contact
    			if (name.getText().equals(""))
    				EmptyNameWarning();
    			else {
    				Contact newContact = new Contact(name.getText(), email.getText(), phone.getText(),
    							street.getText(), city.getText(), zipcode.getText(), country.getText());
    					if (contact == null){ //new contact
    						try {
    							book.get(name.getText()); //search the name in the address book
    							NameExistsWarning(); //if book.get(...) not thrown, means the name exists in the address book
    						} catch (Exception e1){ //the name doesn't appear in the address book, safe to add
    							book.add(newContact);
    							changed = true;
    							localShell.close();
    						}
    				} else { //editing an existing contact
    					if (contact.getName().toLowerCase().equals(name.getText().toLowerCase())){ //staying with the same name (case insensitive)
    						book.modify(newContact);
    						changed = true;
    						localShell.close();
    					} else try { //check if the new name already exists
    						book.get(name.getText());
    						NameExistsWarning();
    					} catch (Exception e2){ //replacing the old contact with the new one (different names)
    						book.delete(contact.getName());
    						book.add(newContact);
    						changed = true;
    						localShell.close();
    					}
    				}
    			}
    		} catch (Exception WillNeverBeThrown){}}}); //end of try-catch wrap
    	
    	
    	
		localShell.open();
		while(!localShell.isDisposed())
			if (!localDisplay.readAndDispatch())
				localDisplay.sleep();
	}
	
	private void newContact(){
		ContactDetails(null);
	}
	
	private void editContact(){
		if (table.getItemCount() == 0){
			NoContactsWarning("edit");
		} else {
			ContactDetails(getContact());
			displayUpdate();
		}
	}
	
	private void deleteContact(){
		if (table.getItemCount() == 0){
			NoContactsWarning("delete");
		} else {
		if (confirmDeleteContact()){
			try { //must wrap with try-catch due to use of class Contact
			book.delete(getContact().getName());
			} catch (Exception e){}
			changed = true;
			displayUpdate();
		}}
	}
	
	private Contact getContact() {
		if (table.getSelectionCount()!=0){
			TableItem selectedItem = table.getSelection()[0];
			String selected = selectedItem.getText(0);
			try { //must wrap with try-catch due to use of class Contact
				return book.get(selected);
			} catch (Exception e){}
		} return null;
	}
	
	//	Confirmations:
	//  **************
	
	private void confirmSaveChanges(){
		if (this.changed) {
			MessageBox saveChanges = new MessageBox(shell, SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			saveChanges.setMessage("The address book has been modified. Would you like to save the changes?");
			saveChanges.setText("Save changes");
			if (saveChanges.open() == SWT.YES) saveBook();
			}
		}
	
	private boolean confirmDeleteContact(){
		MessageBox confirmDelete = new MessageBox(shell, SWT.ICON_QUESTION | SWT.YES | SWT.NO);
		confirmDelete.setMessage("The selected contact will be deleted. Are you sure?");
		confirmDelete.setText("Delete contact");
		if (confirmDelete.open() == SWT.YES) return true;
		else return false;
	}
	
	private void confirmExit(){
		MessageBox confirmExit = new MessageBox(shell, SWT.ICON_QUESTION | SWT.YES | SWT.NO);
		confirmExit.setMessage("Are you sure you want to exit?");
		confirmExit.setText("Exit Address Book");
		if (confirmExit.open() == SWT.YES) shell.close();
	}
	
	//  Warnings and errors:
	//  ********************
	
	private void EmptyNameWarning(){
		MessageBox emptyName = new MessageBox(shell, SWT.ICON_WARNING | SWT.OK);
		emptyName.setMessage("Cannot accept an empty name field.");
		emptyName.setText("Name field empty");
		emptyName.open();
	}
	
	private void NameExistsWarning(){
		MessageBox nameExists = new MessageBox(shell, SWT.ICON_WARNING | SWT.OK);
		nameExists.setMessage("A contact with the given name already exists.");
		nameExists.setText("Name already Exists");
		nameExists.open();
	}
	
	private void FileErrorWarning(String operation){
		MessageBox fileError = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
		fileError.setMessage("Unable to " + operation + " the file.");
		fileError.setText("File error");
		fileError.open();
	}
	
	private void NoContactsWarning(String operation){
		MessageBox noContacts = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
		noContacts.setMessage("Unable to " + operation + ", no contacts in the address book.");
		noContacts.setText("No contacts");
		noContacts.open();
	}
}
