/**
 * representation invariants:
 * @imp_inv maxNumStudents == maxNumStudents(), maxNumStudents == students.length
 * @imp_inv top == numOfStudents() + 1, (top >= -1) && (top < maxNumStudents)
 * @imp_inv units == units(), (units >= 1) && (units <= 5)
 * @imp_inv level == level(), (level >= 1) && (level <= 3)
 * @imp_inv while(i = 0 to top) students[i] != null //every registered student is initialized
 *
 */
public class SimpleCourse implements Course {

	private int maxNumStudents;
	private int top;
	private int units;
	private int level;
	private Student[] students;

	public SimpleCourse(int maxNumStudents, int units, int level) {
		this.maxNumStudents = maxNumStudents;
		students = new Student[maxNumStudents];
		this.units = units;
		this.level = level;
		top = -1;
		
	}
	
	/** 
	 * returns the units of the course
	 * @post $ret == this.units
	 */
	public int units(){
		return this.units;
	}
	
	/**
	 * returns the level of the course
	 * @post $ret == this.level
	 */
	public int level(){
		return this.level;
	}
	
	/**
	 * returns the number of students
	 * @post $ret == this.top + 1
	 */
	public int numOfStudents(){
		return (top + 1);
	}
	
	/**
	 * returns the maximum number of students allowed to be registered to the course
	 * @post $ret == this.maxNumStudents
	 */
	public int maxNumStudents(){
		return this.maxNumStudents;
	}

	/**
	 * returns true iff s is registered to the course
	 * @pre s != null
	 * @post $ret =  s appears in this.students[]
	 */
	public boolean registered(Student s){
		if (this.numOfStudents() == 0) return false;
		else {
			for(int i = 0; i <= top; i++){
				if (s == this.students[i]) return true;
			}
			return false;
		}
	}
	
	//notice that to register or drop a student from a course, changing
	//both the Student and the Course, client should use the student's
	//register and drop methods.
	
	/**
	 * registers s to the course
	 * @pre s != null
	 * @pre this.registered(s) == false
	 * @pre this.numOfStudents() < this.maxNumStudents()
	 * @post this.registered(s) == true
	 * @post this.numOfStudents() == $prev(this).numOfStudents() + 1
	 */
	public void register(Student s) {
		students[++top] = s;
	}
	
	/**
	 * unregisters s from the course
	 * @pre this.registered(s) == true
	 * @post this.registered(s) == false
	 * @post this.numOfStudents() == $prev(this).numOfStudents() - 1
	 */
	public void drop(Student s){
		for(int i = 0; i <= top; i++){
			if (s == students[i]){
				students[i] = students[this.top];
				students[this.top] = null;
				this.top--;
				break;
			}
		}
	}
}
