public interface Student {

	/**
	 * registers the student to course c
	 * @pre c != null
	 * @pre c.registered(this) == false
	 * @pre c.numOfStudents() < c.maxNumStudents()
	 * @pre this.totalUnits() + c.units() <= 10
	 * @post c.registered(this) == true
	 * @post c.numOfStudents() == $prev(c).numOfStudents() + 1
	 * @post this.totalUnits() == $prev(this).totalUnits() + c.units()
	 */
	public void register(Course c);

	/**
	 * unregisteres the student from course c
	 * @pre c.registered(this) == true
	 * @post this.totalUnits() == $prev(this).totalUnits() - c.units()
	 * @post c.registered(this) == false
	 * @post c.numOfStudents() == $prev(c).numOfStudents() - 1 
	 */
	public void drop(Course c);

	/**
	 * returns the number of units of the total courses the student is registered to
	 * @pre s != null
	 * @post ($ret >= 0) && ($ret <= 10)
	 */
	public int totalUnits();

	/**
	 * returns the name of the student
	 * @pre s != null
	 * @post ($ret != null) && ($ret != "") //will not allow a student with an "empty" name
	 */
	public String name();
}
