import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Scanner;

public class TextualAddressBookViewer {

	/**
	 * receives a string and returns an array of length 7 that is the split
	 * of the input string with the delimiter ";"
	 * @pre data != null;
	 * @param data - the input string has no more than 7 substrings separated
	 * by ";" in it.
	 * @return the array as described above
	 */
	private static String[] makeContactInput(String data){
		String[] input = new String[7];
		for(int i = 0; i < 7; i++) input[i] = "";
		String[] toInput = data.split(";");
		int i = 0;
		for(String s:toInput){
			input[i] = s;
			i++;
		}
		return input;
	}
	
	/**
	 * The program receives a filename with orders to operate on an
	 * address book. 
	 * @param args - agrs[0] holds the filename.
	 * @throws Exception - returns error messages by AddressBook exceptions
	 */
	public static void main(String[] args) throws Exception{
	try{
		//create a scanner for the input file
		FileReader in = new FileReader(args[0]);
		Scanner fileScanner = new Scanner(in);
		AddressBook book = null;
		
		while (fileScanner.hasNext()) {
			String nextLine = fileScanner.nextLine();
			char action = nextLine.charAt(0);
			String data;
			if (nextLine.length() > 1) 
				data = nextLine.substring(2);
			else data = "";
			
			switch (action) {
			case 'n':{
				book = new AddressBook();
				System.out.println("NEW: A new address book has been created.");
				break;
			} case 'l':{
				book.load(data);
				System.out.println("LOAD: The file " + data + " has been loaded.");
				break;
			} case 'a':{
				String[] input = makeContactInput(data);
				book.add(new Contact(input));
				System.out.println("ADD: The contact with the name "
									+ input[0] + " has been added.");
				break;
			} case 'p':{
				Iterator<Contact> iter = book.search(data);
				if (iter.hasNext()){
					System.out.println("SEARCH: The contacts with names that start " +
									   "with " + data + ":");
					while(iter.hasNext()) System.out.println("- " + iter.next());
				} else System.out.println("SEARCH: No contacts with the prefix "
										   + data + " in the address book");
				break;
			} case 'x':{
				Iterator<Contact> iter = book.getContacts();
				if (iter.hasNext()){
					System.out.println("GET-CONTACTS: All the contacts in the " +
							"address book:");
					while(iter.hasNext()) System.out.println("- " + iter.next());
				} else System.out.println("GET-CONTACTS: No contacts " +
										  "in the address book");
				break;
			} case 'd':{
				book.delete(data);
				System.out.println("DELETE: The contact with the name "
									+ data + " has been deleted.");
				break;
			} case 'm':{
				String[] input = makeContactInput(data);
				book.modify(new Contact(input));
				System.out.println("MODIFY: The contact with the name "
									+ input[0] + " has been modified.");
				break;
			} case 's':{
				book.save(data);
				System.out.println("SAVE: All contacts had been saved to the file " +
									data);
				break;
			}
			}
		}
	} catch (EmptyNameException e){
		System.out.println("EmtyNameException: wrong/no name has been given");
	} catch (ContactAlreadyExistsException e){
		System.out.println("ContactAlreadyExistsException: a contact with the " +
				"given name already exists");
	} catch (ContactDoesNotExistsException e){
		System.out.println("ContactDoesNotExistsException: a contact with the " +
				"given name does not exists");
	} catch (IOException e){
		System.out.println("IOException: input/output error");
	} catch (Exception e){
		System.out.println("Error!");
	}
	}
}
