package turtle;

import il.ac.tau.cs.software1.simple_io.LineInput;
import il.ac.tau.cs.software1.turtle.Turtle;

public class TurtleDrawing {
	
	/**
	 * 
	 * @pre - tr is an initialized Turtle
	 * 		- tr.isTailDown == true
	 * @post - draws the figure:
	 * 			 ^
	 * 			^ ^
	 * 		   ^   ^
	 * 		  ^		^
	 * 		 |-------|
	 * 		 |\     /|
	 * 		 | \   / |
	 * 		 |  \ /  |
	 * 		 |	/ \  |
	 * 		 | /   \ |
	 * 		 |/		\|
	 * 		 ---------
	 * with tr at the bottom-right corner in the end of the drawing, facing right
	 * (with regard to the base of the figure).
	 * @param tr - a Turtle
	 */
	private static void drawFigure(Turtle tr){
		tr.turnLeft(90);
		tr.moveForward(50);
		tr.turnRight(90);
		tr.moveForward(50);
		tr.turnRight(90);
		tr.moveForward(50);
		tr.turnRight(135);
		tr.moveForward(Math.sqrt(5000));
		tr.turnRight(75);
		tr.moveForward(50);
		tr.turnRight(120);
		tr.moveForward(50);
		tr.turnRight(75);
		tr.moveForward(Math.sqrt(5000));
		tr.turnLeft(135);
		tr.moveForward(50);
	}
	
	/**
	 * @pre - 360 % numOfShapes == 0
	 * @post - draws the figure shown on drawFigure(Turtle tr) numOfShapes times,
	 * while rotating so that at the end a circle of the above shape will be formed.
	 * No turtle will be seen in the drawing.
	 * @param numOfShapes - an integer that divides 360
	 */
	public static void drawSequence(int numOfShapes){
		Turtle myTurtle = new Turtle();
		
		myTurtle.tailDown();
		for(int i = 0; i < numOfShapes; i++){
			drawFigure(myTurtle);
			myTurtle.turnRight(360 / numOfShapes);
		}
		myTurtle.hide();
	}
	
	public static void main(String[] args) {
		int input;
		System.out.print("Enter a number that divides 360: ");
		input = LineInput.readInt();
		drawSequence(input);
	}
}
