
public abstract class TrenaryOp implements Expression{
	private Expression operand1;
	private Expression operand2;
	private Expression operand3;
	
	/**
	 * @pre exp1, exp2, exp3 != null
	 */
	public TrenaryOp(Expression exp1, Expression exp2, Expression exp3){
		this.operand1 = exp1;
		this.operand2 = exp2;
		this.operand3 = exp3;
	}
	
	/**
	 * @pre exp1, exp2, exp3 != null
	 * @post $ret = the evaluation of the TrenaryOp expression
	 */
	abstract protected Double operator(Expression exp1, Expression exp2, Expression exp3);
	
	/**
	 * @post $ret = the string representation of the first operator
	 */
	abstract protected String operator1ToString();
	
	/**
	 * @post $ret = the string representation of the second operator
	 */
	abstract protected String operator2ToString();
	
	/**
	 * @post $ret = operator(operand1, operand2, operand3)
	 */
	public Double eval(){
		return operator(operand1, operand2, operand3);
	}
	
	/**
	 * @post $ret = the string representation of the expression
	 */
	public String toString(){
		String result = "";
		result += "(" + operand1 + ") " + operator1ToString() +
				  " (" + operand2 + ") " + operator2ToString() +
				  " (" + operand3 + ")";
		return result;
	}
}
