
public abstract class CompositeSortedIterator<T extends Comparable<T>> implements SortedIterator<T>{
	
	protected SortedIterator<T> aIter;
	protected SortedIterator<T> bIter;
	protected T next;
	protected T aElement;
	protected T bElement;
	
	/**
	 * Constructor
	 * an abstract iterator for some implementations of CompositeSortedSet iterators.
	 * will be used for Merged, Difference and Unified iterators.
	 * @pre a, b != null
	 */
	public CompositeSortedIterator(SortedIterator<T> a, SortedIterator<T> b){
		aIter = a;
		bIter = b;
		advanceAElement();
		advanceBElement();
		next = checkNext();
	}
	
	abstract protected T checkNext();
	
	/**
	 * @post aIter.hasNext() $implies aElement = aIter.next()
	 * 						 else $implies aElement = null
	 */
	protected void advanceAElement(){
		if (aIter.hasNext()) aElement = aIter.next();
		else aElement = null;
	}
	
	/**
	 * @post bIter.hasNext() $implies bElement = bIter.next()
	 * 						 else $implies bElement = null
	 */
	protected void advanceBElement(){
		if (bIter.hasNext()) bElement = bIter.next();
		else bElement = null;
	}
	
	/**
	 * @pre this.hasNext()
	 * @post $ret = the next element in this set 
	 */
	public T next(){
		T result = next;
		next = checkNext();
		return result;
	}
	

	/**
	 * @post $ret = (next != null)
	 */
	public boolean hasNext(){
		return (next != null);
	}
	
	public void remove(){}
}
