
public abstract class CompositeSortedSet<T extends Comparable<T>> implements SortedSet<T>{
	
	private int size = -1;
	private T minimum = null;
	private T maximum = null;
	
	/**
	 * @post $ret = the size of this set
	 */
	public int size(){
		if (size == -1){
			int result = 0;
			for(SortedIterator<T> iter = this.iterator(); iter.hasNext();){
				result++;
				iter.next();
			}
			size = result;
		}
		return size;
	}
	
	/**
	 * @post $ret = true iff this is an empty set, false otherwise
	 */
	public boolean isEmpty(){
		int result = this.size();
		return (result == 0);
	}
	
	/**
	 * @post this.isEmpty? $implies $ret = null
	 * 						else $implies $ret = the minimal element of this set
	 */
	public T getMinimum(){
		if (minimum == null) {
			if (!this.isEmpty()){
				SortedIterator<T> iter = this.iterator();
				minimum = iter.next();
			}
		}
		return minimum;
	}
	
	/**
	 * @post this.isEmpty? $implies $ret = null
	 * 						else $implies $ret = the maximal element of this set
	 */
	public T getMaximum(){
		if (maximum == null) {
			if (!this.isEmpty()){
				T result = null;
				for(SortedIterator<T> iter = this.iterator(); iter.hasNext();)
					result = iter.next();
				maximum = result;
			}
		}
		return maximum;
	}
}
