import java.util.*;
public class SimpleSortedSet implements SortedSet<Integer> {
	
	private Integer[] values;
	
	/**
	 * Constructs a new SimpleSortedSet with the given int array as elements of the set
	 * the elements of the set are all of type Integer, and are determined at construction
	 * of the set.
	 */
	public SimpleSortedSet (int[] values) {
		TreeMap<Integer,Integer> map = new TreeMap<Integer,Integer> ();
		for (int i = 0; i < values.length; i++) {
			Integer next = new Integer(values[i]);
			map.put(next,next);
		}
		int newSize = map.size();
		this.values = new Integer[newSize];
		Iterator<Integer> iter = map.keySet().iterator();
		for (int i = 0; i < newSize; i++) {
			this.values[i] = iter.next();
		}
	}
	
	/**
	 * @return the power of this set, i.e the number of elements in the set
	 */
	public int size() {
		return values.length;
	}
	
	/**
	 * @return a boolean expression if this set contains the given element
	 */
	public boolean contains (Integer element) {
		return Arrays.binarySearch(values,(Integer)element) > -1;
	}
	
	/**
	 * @return true iff the set is empty
	 */
	public boolean isEmpty () {
		return values.length == 0;
	}
	
	/**
	 * @return the mimimum element in this set
	 * returns null if the set is empty
	 */
	public Integer getMinimum() {
		if (values.length == 0) {
			return null;
		}
		return values[0];
	}
	
	/**
	 * @return the maximum element in this set
	 * returns null if the set is empty
	 */
	public Integer getMaximum() {
		if (values.length == 0) {
			return null;
		}
		return values[values.length - 1];
	}
	
	/**
	 * returns an Iterator for the elements of this set
	 */
	public SortedIterator<Integer> iterator() {
		return new SimpleSortedIterator (Arrays.asList(values).iterator());
	}
	
	
	public static void main (String[] args) {
		SimpleSortedSet seta = new SimpleSortedSet(new int[] {1,2,3,4});
		SimpleSortedSet setb = new SimpleSortedSet(new int[] {3,4,5,6});
		SortedSet setC = new MergedSortedSet(seta,setb);
		Iterator iter = setC.iterator();
		while (iter.hasNext()) {
			System.out.println(iter.next());
		}
		System.out.println("size = " + setC.size());
		System.out.println("maximum = " + setC.getMaximum());
		System.out.println("minimum = " + setC.getMinimum());
		System.out.println("empty ? = " + setC.isEmpty());
	}
	
}

// A default implementation for SortedIterator
// decorates an iterator that is known to be sorted
class SimpleSortedIterator implements SortedIterator<Integer> {
	private Iterator<Integer> iter;
	
	public SimpleSortedIterator (Iterator<Integer> iter) {
		this.iter = iter;
	}
	
	public boolean hasNext() {
		return iter.hasNext();
	}
	
	public Integer next() {
		return iter.next();
	}
	
	public void remove() {
		iter.remove();
	}
}



