/**
 *  An interface for a set of elements, whose iterator returns the elements
 *  in ascending order.
 *  The elements are a Comparable type
 */
public interface SortedSet<T extends Comparable<T>> {
	
	/**
	 * Return the power of this set, i.e the number of elements in the set.
	 * @return the power of this set, i.e the number of elements in the set
	 */
	public int size();

	/**
	 * Return a boolean expression if this set contains the given element.
	 * @return a boolean expression if this set contains the given element
	 */
	public boolean contains (T element);
	
	/**
	 * Return true iff the set is empty.
	 * @return true iff the set is empty
	 */
	public boolean isEmpty ();
	
	/**
	 * Return the mimimum element in this set.
	 * @return the mimimum element in this set
	 */
	public T getMinimum();
	
	/**
	 * Return the maximum element in this set.
	 * @return the maximum element in this set
	 */
	public T getMaximum();
	
	/**
	 * Returns an Iterator for the elements of this set.
	 * @return	An Iterator for the elements of this set
	 */
	public SortedIterator<T> iterator();
}
