import java.lang.reflect.*;
import java.util.*;
import java.io.*;

// A test for MergedSortedSet,UnifiedSortedSet and DifferenceSortedSet.
// The test output will be written into a file named "Output.txt"
public class Test {
	
	// File writer
	private static PrintWriter writer;
	
	
	// Data to use
	private static final int [][] data = new int[][] {
		{1,2,3,4,5,6,8,9,10},
		{3,4,5,6,8},
		{0,1,2,3,4},
		{5,6,8,9,10,11},
		{1,2,3,4,5,6,8,10},
		{2,4,8,12,13,14,35},
		{},
		{-23,22,234}
	};
	
	private static final SimpleSortedSet[] sortedSets;
	
	//  the SortedSet array
	static {
		sortedSets = new SimpleSortedSet[data.length];
		for (int i = 0; i < sortedSets.length; i++) {
			sortedSets[i] = new SimpleSortedSet(data[i]);
		}
	}
	
	
	// The type of composite sorters to be tested
	private static final  Class[] compositeSorters = {
		MergedSortedSet.class,
			UnifiedSortedSet.class,
			DifferenceSortedSet.class
	};
	
	// Main entry for this program
	public static void main (String[] args) {
		try {
			writer = new PrintWriter(new FileOutputStream("Output.txt"));
		}catch(IOException ioe) {
			ioe.printStackTrace();
		}
		
		// Runs a test for each composite sorter defined
		for (int j = 0; j < compositeSorters.length; j++) {
			writer.println("\n********************************");
			writer.println("Testing class  " + compositeSorters[j].getName());
			writer.println("\n********************************");
			SortedSet setA = sortedSets[0];
			for (int i = 1; i < sortedSets.length; i++) {
				SortedSet testSet = null;
				try {
					
					// get the constructor that receives 2 sorted set objects
					Constructor constructor = compositeSorters[j].getConstructor(
						new Class [] {SortedSet.class, SortedSet.class});
					
					// the arguments for the constructor
					Object[] newSets = new Object[] {setA,sortedSets[i]};
					
					// create a new instance for the sorted set
					testSet =  (SortedSet) constructor.newInstance(newSets);
				} catch (Exception e) {
					writer.println("failed to create test set");
				}
				printTestDetails(data[0],data[i]);
				doTest(testSet);
			};
		};
		writer.flush();
		writer.close();
		System.out.println("Test is done. The output is in the Output.txt file.");
	}
	
	// Prints the 2 data arrays we are going to test
	private static void printTestDetails(int[] set1, int[] set2) {
		writer.println ("\n\nData to test:");
		writer.print("{ ");
		for(int i=0; i<set1.length; i++) {
			writer.print(set1[i]+" ");
		}
		writer.println("}");
		writer.print("{ ");
		for(int i=0; i<set2.length; i++) {
			writer.print(set2[i]+" ");
		}
		writer.println("}");
		writer.println();
	}
	
	
	// Tests the SortedSet
	private static void doTest(SortedSet set) {
		writer.println("test results:");
		Iterator iter = set.iterator();
		writer.print("{ ");
		while (iter.hasNext()) {
			writer.print(iter.next()+" ");
		}
		writer.println("}");
		writer.println("size = " + set.size());
		writer.println("maximum = " + set.getMaximum());
		writer.println("minimum = " + set.getMinimum());
		writer.println("empty ? = " + set.isEmpty());
		writer.println("contains '4'? = " + set.contains(new Integer(4)));
		writer.println("contains '6'? = " + set.contains(new Integer(6)));
		writer.println("end of test");
		writer.println();
	}
}
