import java.util.Iterator;
import java.util.NoSuchElementException;


public class prefixIterator implements Iterator<Contact>{
	
	private String prefix;
	private Iterator<Contact> iter;
	private Contact next;
	
	/**
	 * Constructor: creates a new prefixIterator, an iterator for a collection
	 * of Contacts, that runs over the contacts with names starting with the given
	 * prefix
	 * @param pre - the prefix to look the contacts by
	 * @param originalIter - an iterator for the original contacts collection
	 */
	public prefixIterator(String pre, Iterator<Contact> originalIter){
		prefix = pre.toLowerCase();
		iter = originalIter;
		next = checkNext();
	}
	
	/**
	 * @pre this.hasNext()
	 * @return the next contact in the address book that its name starts with the prefix
	 */
	public Contact next() throws NoSuchElementException {
		if (next == null) throw new NoSuchElementException();
		Contact temp = next;
		next = checkNext();
		return temp;
	}
	
	/**
	 * @return (next == null)
	 */
	public boolean hasNext(){
		return (next != null);
	}
	
	public void remove(){}
	
	/**
	 * @return the next contact with a name starting with the prefix
	 */
	private Contact checkNext(){
		if (iter.hasNext()){
			Contact temp = iter.next();
			if (temp.getName().toLowerCase().startsWith(prefix))
				return temp;
			else return checkNext();
		} else return null;
	}
}
